//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using Microsoft.VisualStudio.TestTools.UnitTesting;
using OrderManagement.FunctionalTests.UseCaseLayer.Actions;
using OrderManagement.FunctionalTests.UseCaseLayer.Assertions;
using OrderManagement.FunctionalTests.UseCaseLayer.Entities;

namespace OrderManagement.FunctionalTests
{
	/// <summary>
	/// Tests for user stories related to saving an order.
	/// </summary>
	[TestClass]
	public class SaveOrderTests
	{
		private static ITestDataRepository dataRep;

		public SaveOrderTests()
		{
		}


		[TestInitialize]
		public void TestInit()
		{
			dataRep = new TestDataRepositoryProvider().GetDefaultTestDataRepository();
			TestInfrastructure.StartWebServer();
			TestInfrastructure.CreateNewBrowserInstance();
			SubmitOrderActions.NavigateToDefaultView();
		}


		[TestCleanup]
		public void TestCleanup()
		{
			TestInfrastructure.CloseBrowserInstance();
			TestInfrastructure.StopWebServer();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and order details and saves the order.
		/// </UserStory>
		[TestMethod]
		public void SaveOrderFunctionalityTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and order details and saves the order. The user repeats the same procedure multiple times
		/// in the same session.
		/// </UserStory>
		[TestMethod]
		public void SaveMultipleOrdersInTheSameSessionTest()
		{
			Order inputOrder = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order1");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.CreateOrder(inputOrder);

			inputOrder = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order2");
			SubmitOrderActions.CreateOrder(inputOrder);

			inputOrder = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order3");
			SubmitOrderActions.CreateOrder(inputOrder);

			Order expectedOrderValuesToAssert = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order1");
			SubmitOrderAssertions.AssertSubmitOrderSuccess(expectedOrderValuesToAssert);

			expectedOrderValuesToAssert = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order2");
			SubmitOrderAssertions.AssertSubmitOrderSuccess(expectedOrderValuesToAssert);

			expectedOrderValuesToAssert = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order3");
			SubmitOrderAssertions.AssertSubmitOrderSuccess(expectedOrderValuesToAssert);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then deletes some order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderAfterDeletingOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderAfterDeletingSomeOrderDetailsTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}

			SubmitOrderActions.DeleteOrderLineItem(inputOrder.Details[0]);
			inputOrder.Details.Remove(inputOrder.Details[0]);

			SubmitOrderActions.DeleteOrderLineItem(inputOrder.Details[1]);
			inputOrder.Details.Remove(inputOrder.Details[1]);

			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and adds multiple products to the order details; then edits some order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderAfterEditingOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			Order newInputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order WithNewOrderDetails");


			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}

			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[0], newInputOrder.Details[0]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[1], newInputOrder.Details[1]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[3], newInputOrder.Details[3]);

			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(newInputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products to the order details by searching for the products. Then the user saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderAfterAddingOrderDetailsThroughProductSearchTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			Order newInputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order WithNewOrderDetails");


			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}

			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[0], newInputOrder.Details[0]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[1], newInputOrder.Details[1]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[3], newInputOrder.Details[3]);

			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(newInputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details. The user then edits some order details by searching for products 
		/// and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderAfterEditingOrderDetailsThroughProductSearchTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			Order newInputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order WithNewOrderDetails");


			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}

			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[0], newInputOrder.Details[0]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[1], newInputOrder.Details[1]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[3], newInputOrder.Details[3]);

			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(newInputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and saves the order, without adding order details.
		/// </UserStory>
		[TestMethod]
		public void SaveOrderWithNoOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and adds multiple products in the order details.
		/// Then the user saves the order, multiple times without modifying the order.
		/// </UserStory>
		[TestMethod]
		public void SaveOrderMultipleTimesWithoutEditingTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and adds multiple products in the order details.
		/// The user saves the order multiple times after modifying the order each time. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderMultipleTimesAfterEditingTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter order name and city.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes order name and city.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithEmptyOrderNameAndCityTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not select the approver and state.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user selects empty approver and state.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithEmptyApproverAndStateTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter customer and zip.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes customer and zip.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithEmptyCustomerAndZipTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithEmptyStreetTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderAllowedWithEmptyDescriptionTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidCustomerTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithSpecialCharactersInCustomerTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidZipTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidCityTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidOrderNameTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidDescriptionTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderAllowedWithSpecialCharactersInDescriptionTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidProductSKUTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrderNotAllowedWithInvalidQuantityTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void CancelSaveOrderTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void CancelSavingOrderDetailsAfterSavingOrderInformationTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void CancelSavingOrderAfterSavingAndEditingOrderTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void AddOrderInformation_SaveOrder_AddOrderDetails_SaveOrder_ScenarioFlowTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void Login_SaveOrder_Logout_MultipleTimes_ScenarioFlowTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrder_CancelSubmit_EditOrder_SaveOrder_ScenarioFlowTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information.
		/// The user does not enter the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// The user logs in, fills the order information.
		/// The user deletes the street address.
		/// The user then adds multiple products in the order details and saves the order. 
		/// </UserStory>
		[TestMethod]
		public void SaveOrder_CancelSubmit_EditOrder_CancelSaveOrder_ScenarioFlowTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}
	}
}