//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using Microsoft.VisualStudio.TestTools.UnitTesting;
using OrderManagement.FunctionalTests.UseCaseLayer.Actions;
using OrderManagement.FunctionalTests.UseCaseLayer.Assertions;
using OrderManagement.FunctionalTests.UseCaseLayer.Entities;

namespace OrderManagement.FunctionalTests
{
	/// <summary>
	/// Tests to test the Submit Order functionality.
	/// </summary>
	[TestClass]
	public class SubmitOrdersTests
	{
		private static ITestDataRepository dataRep;

		public SubmitOrdersTests()
		{
		}


		[TestInitialize]
		public void TestInit()
		{
			dataRep = new TestDataRepositoryProvider().GetDefaultTestDataRepository();
			TestInfrastructure.StartWebServer();
			TestInfrastructure.CreateNewBrowserInstance();
			SubmitOrderActions.NavigateToDefaultView();
		}


		[TestCleanup]
		public void TestCleanup()
		{
			TestInfrastructure.CloseBrowserInstance();
			TestInfrastructure.StopWebServer();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and order details and submits the order.
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderFunctionalityTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and order details and submits the order. The user repeats the same procedure multiple times
		/// in the same session.
		/// </UserStory>
		[TestMethod]
		public void SubmitMultipleOrdersInTheSameSessionTest()
		{
			// Create the order from a template
			Order inputOrder = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order1");

			// Get a specific user's information
			UserDetails user = dataRep.GetUserDetails("User1");

			// Do the Login 
			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);

			// Create and Submit an Order
			SubmitOrderActions.CreateOrder(inputOrder);

			// Create and Submit another Order
			inputOrder = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order2");
			SubmitOrderActions.CreateOrder(inputOrder);

			// Create and Submit another Order
			inputOrder = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order3");
			SubmitOrderActions.CreateOrder(inputOrder);

			// Verify that the orders were properly added to the respository
			Order expectedOrderValuesToAssert = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order1");
			SubmitOrderAssertions.AssertSubmitOrderSuccess(expectedOrderValuesToAssert);

			expectedOrderValuesToAssert = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order2");
			SubmitOrderAssertions.AssertSubmitOrderSuccess(expectedOrderValuesToAssert);

			expectedOrderValuesToAssert = dataRep.GetOrder("MultipleOrdersInSameSessionTest Order3");
			SubmitOrderAssertions.AssertSubmitOrderSuccess(expectedOrderValuesToAssert);

			// Logout
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then deletes some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderAfterDeletingSomeOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderAfterDeletingSomeOrderDetailsTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}

			SubmitOrderActions.DeleteOrderLineItem(inputOrder.Details[0]);
			inputOrder.Details.Remove(inputOrder.Details[0]);

			SubmitOrderActions.DeleteOrderLineItem(inputOrder.Details[1]);
			inputOrder.Details.Remove(inputOrder.Details[1]);

			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and adds multiple products to the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderAfterEditingSomeOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			Order newInputOrder = dataRep.GetOrder("SubmitOrderAfterEditingSomeOrderDetailsTest Order WithNewOrderDetails");


			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}

			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[0], newInputOrder.Details[0]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[1], newInputOrder.Details[1]);
			SubmitOrderActions.EditOrderLineItem(inputOrder.Details[3], newInputOrder.Details[3]);

			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(newInputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderWithEmptyOrderDescriptionTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderWithEmptyOrderDescriptionAndCityTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderWithEmptyStateAndZipTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderWithEmptyOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderAfterGoingBackToEditTheOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}


		/// <UserStory>
		/// The user logs in, fills the order information and add multiple products in the order details; then edits some order details and submits the order. 
		/// </UserStory>
		[TestMethod]
		public void SubmitOrderAfterGoingBackToDeleteTheOrderDetailsTest()
		{
			Order inputOrder = dataRep.GetOrder("SubmitOrderFunctionalityTest Order");
			UserDetails user = dataRep.GetUserDetails("User1");

			SubmitOrderActions.NavigateToDefaultView();
			LoginActions.Login(user);
			SubmitOrderActions.StartCreateOrder();
			SubmitOrderActions.AddOrderInfo(inputOrder);
			foreach (OrderDetails details in inputOrder.Details)
			{
				SubmitOrderActions.AddOrderLineItem(details);
			}
			SubmitOrderActions.AddOrderDetails();
			SubmitOrderActions.SubmitOrder();

			SubmitOrderAssertions.AssertSubmitOrderSuccess(inputOrder);
			LoginActions.Logout();
		}
	}
}