//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RealTimeSearchQuickstart.FunctionalTests.Pages;
using RealTimeSearchQuickstart.FunctionalTests.Actions;
using RealTimeSearchQuickstart.FunctionalTests.Assertions;
using System.Threading;
using System.IO;


namespace RealTimeSearchQuickstart.FunctionalTests
{
    /// <summary>
    /// Tests for the testing the page structure and design.
    /// </summary>    
    [TestClass]
    public class PageDesignTests
    {
        private SearchCustomerPage searchPage;


        /// <summary>
        /// Test Initialization:
        ///     Create a new browser
        ///     Browse to the search customer page
        ///     Validate whether the search customer page has been loaded
        ///     Set the target page for the search customer actions and assertions
        /// </summary>
        [TestInitialize]
        public void TestInit()
        {
            TestInfrastructure.CreateNewBrowserInstance();
            searchPage = new SearchCustomerPage();
            searchPage.Navigate();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation());
            SearchCustomerPageActions.TargetPage = searchPage;
            SearchCustomerPageAssertions.TargetPage = searchPage;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            TestInfrastructure.CloseBrowserInstance();
        }


        /// <summary>
        /// To test that the search button is present in the search page.
        /// </summary>
        [TestMethod]
        public void SearchButtonExistsInSearchPageTest()
        {
            Assert.IsTrue(searchPage.SearchButton.Exists, "Search Button does not exist");
        }


        /// <summary>
        /// To test that the search fields of companyname, state, postal code and city exist in the search page.
        /// </summary>
        [TestMethod]
        public void SearchFieldsExistInSearchPageTest()
        {
            Assert.IsTrue(searchPage.CompanyNameBox.Exists, "Company Name search field does not exist in the search page.");
            Assert.IsTrue(searchPage.CityBox.Exists, "City search field does not exist.");
            Assert.IsTrue(searchPage.StateList.Exists, "State search field does not exist.");
            Assert.IsTrue(searchPage.ZipCodeBox.Exists, "PostalCode search field does not exist");            
        }

        /// <summary>
        /// To test that the grid has the right columns.
        /// </summary>
        [TestMethod]
        public void GridStructureTest()
        {
            SearchCustomerPageActions.SetCity("A");
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            searchPage.ResultsGrid.Initialize();
            Assert.IsTrue(searchPage.ResultsGrid.Rows[0]["CustomerID"].Exists, "The search results does not show the customer id.");
            Assert.IsTrue(searchPage.ResultsGrid.Rows[0]["CompanyName"].Exists, "The search results does not show the company name.");
            Assert.IsTrue(searchPage.ResultsGrid.Rows[0]["Address"].Exists, "The search results does not show the address.");
            Assert.IsTrue(searchPage.ResultsGrid.Rows[0]["City"].Exists, "The search results does not show the city.");
            Assert.IsTrue(searchPage.ResultsGrid.Rows[0]["State"].Exists, "The search results does not show the state.");
            Assert.IsTrue(searchPage.ResultsGrid.Rows[0]["ZipCode"].Exists, "The search results does not show the zip code.");
        }




        /// <summary>
        /// To test that the links in the navigation sidebar point to the expected pages.
        /// </summary>
        [TestMethod]
        public void NavigationLinksTest() 
        {
            Assert.IsTrue(searchPage.ValidateLocation(),"The expected page is not the search page");
            searchPage.HomeLink.Click();
            Assert.IsTrue(searchPage.ValidateLocation(), "The home link did not redirect to the search page.");
            searchPage.CustomersLink.Click();

            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            DefaultPage customerPage = new DefaultPage();
            Assert.IsTrue(customerPage.ValidateLocation(), "The customers link did not redirect to the customer page.");
            customerPage.SearchLink.Click();

            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation(), "The search link did not redirect to the search page");
        }



        /// <summary>
        /// To test that the paging links are present
        /// </summary>
        [TestMethod]
        public void PagingLinksExistInSearchPageTest()
        {
            SearchCustomerPageActions.SetCity("A");
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            searchPage.ResultsGrid.Initialize();
            Assert.IsTrue(searchPage.ResultsGrid.NextPageLink.Exists, "Next Page Link does not exist");
            Assert.IsTrue(searchPage.ResultsGrid.LastPageLink.Exists, "Last Page Link does not exist");
            

            searchPage.ResultsGrid.NextPageLink.Click();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            searchPage.ResultsGrid.Initialize();
            Assert.IsTrue(searchPage.ResultsGrid.FirstPageLink.Exists, "First Page Link does not exist");
            Assert.IsTrue(searchPage.ResultsGrid.PreviousPageLink.Exists, "Previous Page Link does not exist");
        }

    }
}
