//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RealTimeSearchQuickstart.FunctionalTests.Pages;
using RealTimeSearchQuickstart.FunctionalTests.Actions;
using RealTimeSearchQuickstart.FunctionalTests.Assertions;
using System.Threading;
using System.IO;

namespace RealTimeSearchQuickstart.FunctionalTests
{

    /// <summary>
    /// Tests for the paging search results functionality.
    /// </summary>
    [TestClass]
    public class PagingTests
    {
        private SearchCustomerPage searchPage;


        /// <summary>
        /// Test Initialization:
        ///     Create a new browser
        ///     Browse to the search customer page
        ///     Validate whether the search customer page has been loaded
        ///     Set the target page for the search customer actions and assertions
        /// </summary>
        [TestInitialize]
        public void TestInit()
        {
            TestInfrastructure.CreateNewBrowserInstance();
            searchPage = new SearchCustomerPage();
            searchPage.Navigate();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation());
            SearchCustomerPageActions.TargetPage = searchPage;
            SearchCustomerPageAssertions.TargetPage = searchPage;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            TestInfrastructure.CloseBrowserInstance();
        }


        /// <summary>
        /// To test that the search results are refreshed when the user is on the second page of the 
        /// search results and the search button is clicked.
        /// </summary>
        [TestMethod]
        public void RefreshSearchResultsWhenSearchButtonIsClickedOnSecondPageTest()
        {
            SearchCustomerPageActions.SetCity("A");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");

            SearchCustomerPageActions.GoToNextPageOfSearchResults();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("23", "Folies gourmandes");

            SearchCustomerPageActions.ClickSearchButton();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");
        }



        /// <summary>
        /// To test that the next page link works
        /// </summary>
        [TestMethod]
        public void NextPageLinkTest()
        {
            SearchCustomerPageActions.SetCity("A");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");

            SearchCustomerPageActions.GoToNextPageOfSearchResults();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("23", "Folies gourmandes");

            SearchCustomerPageActions.GoToNextPageOfSearchResults();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("39", "K" + "\u00F6" + "niglich Essen");
        }


        /// <summary>
        /// To test that the previous page link works
        /// </summary>
        [TestMethod]
        public void PreviousPageLinkTest()
        {
            SearchCustomerPageActions.SetCity("A");

            SearchCustomerPageActions.GoToNextPageOfSearchResults();
            SearchCustomerPageActions.GoToNextPageOfSearchResults();
            SearchCustomerPageActions.GoToPreviousPageOfSearchResults();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("23", "Folies gourmandes");
        }



        /// <summary>
        /// To test that the last page link works
        /// </summary>
        [TestMethod]
        public void LastPageLinkTest()
        {
            SearchCustomerPageActions.SetCity("A");

            SearchCustomerPageActions.GoToLastPageOfSearchResults();
           
            SearchCustomerPageAssertions.AssertSearchResultsCount(4);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("91", "Wolski  Zajazd");
        }



        /// <summary>
        /// To test that the first page link works
        /// </summary>
        [TestMethod]
        public void FirstPageLinkTest()
        {
            SearchCustomerPageActions.SetCity("A");        

            SearchCustomerPageActions.GoToNextPageOfSearchResults();
            SearchCustomerPageActions.GoToNextPageOfSearchResults();
            SearchCustomerPageActions.GoToNextPageOfSearchResults();

            SearchCustomerPageActions.GoToFirstPageOfSearchResults();
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");
        }


               
        /// <summary>
        /// Each page shows a maximum of 10 results. To test that the total number of results and
        /// number of pages of the search results are appropriately proportional.
        /// </summary>
        [TestMethod]
        public void NumberofPagesToNumberofResultsRatioTest()
        {
            SearchCustomerPageActions.SetCity("A");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);

            for (int pageIndex = 0; pageIndex < 5; pageIndex++)
            {
                SearchCustomerPageActions.GoToNextPageOfSearchResults();
                SearchCustomerPageAssertions.AssertSearchResultsCount(10);
                SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            }

            SearchCustomerPageActions.GoToNextPageOfSearchResults();
            SearchCustomerPageAssertions.AssertSearchResultsCount(4);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);          
        }
        
    }
}
