//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RealTimeSearchQuickstart.FunctionalTests.Pages;
using RealTimeSearchQuickstart.FunctionalTests.Actions;
using RealTimeSearchQuickstart.FunctionalTests.Assertions;
using System.Threading;
using System.IO;

namespace RealTimeSearchQuickstart.FunctionalTests
{
    /// <summary>
    /// Tests for the search customer by city functionality.
    /// </summary>
    [TestClass]
    public class SearchCustomerByCityTests
    {
        private SearchCustomerPage searchPage;


        /// <summary>
        /// Test Initialization:
        ///     Create a new browser
        ///     Browse to the search customer page
        ///     Validate whether the search customer page has been loaded
        ///     Set the target page for the search customer actions and assertions
        /// </summary>
        [TestInitialize]
        public void TestInit()
        {
            TestInfrastructure.CreateNewBrowserInstance();
            searchPage = new SearchCustomerPage();
            searchPage.Navigate();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation());
            SearchCustomerPageActions.TargetPage = searchPage;
            SearchCustomerPageAssertions.TargetPage = searchPage;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            TestInfrastructure.CloseBrowserInstance();
        }

       

        /// <summary>
        /// To test that the proper search results are fetched when a single character is entered in the 
        /// city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithASingleCharacterInCityTest()
        {
            SearchCustomerPageActions.SetCity("A");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");
            
        }



        /// <summary>
        /// To test that the proper search results are fetched when a search text is entered in the 
        /// city field and the search button is clicked.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithCityByClickingSearchButtonTest()
        {
            SearchCustomerPageActions.SetCity("AN");
            SearchCustomerPageActions.ClickSearchButton();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(40);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
        }


        /// <summary>
        /// To test that the proper search results are fetched when two characters are entered in the 
        /// city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithTwoCharactersInCityTest()
        {
            SearchCustomerPageActions.SetCity("A");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);

            SearchCustomerPageActions.SetCity("AN");
                       
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(40);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
        }


        /// <summary>
        /// To test that the proper search results are fetched when multiple characters are entered in the 
        /// city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithMultipleCharactersInCityTest()
        {
            SearchCustomerPageActions.SetCity("a");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);

            SearchCustomerPageActions.SetCity("an");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(40);

            SearchCustomerPageActions.SetCity("and");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(26);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            
        }



        /// <summary>
        /// To test that the search functionality works as expected when special characters are used in 
        /// the city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithSpecialCharactersInCityTest()
        {
            SearchCustomerPageActions.SetCity("and"); 
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(26);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");

            SearchCustomerPageActions.SetCity(String.Empty);
            Thread.Sleep(TestInfrastructure.DefaultTimeout);

            foreach (string inputWithSpecialCharacters in TestInfrastructure.EmbedSpecialCharactersInString("and"))
            {
                SearchCustomerPageActions.SetCity(inputWithSpecialCharacters);
                SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
                SearchCustomerPageAssertions.AssertErrorMessageIsDisplayed("Invalid search query string");
            }

            SearchCustomerPageActions.SetCity(String.Empty);
            SearchCustomerPageActions.SetCity(@"'and");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();            
        }


        /// <summary>
        /// To test that the search functionality works as expected when unicode characters are used in 
        /// the city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithUnicodeCharactersInCityTest()
        {
            SearchCustomerPageActions.SetCity("\u00F6");
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }


        /// <summary>
        /// To test that no search results are returned when only blankspaces are entered in the city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithOnlyBlankSpacesInCityTest()
        {
            SearchCustomerPageActions.SetCompanyName("    ");
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }


        /// <summary>
        /// To test that search functionality works as expected when blankspaces are present in the search text
        /// in the city field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithBlankSpacesInSearchTextInCityTest()
        {
            SearchCustomerPageActions.SetCity("       and");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(26);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");

            SearchCustomerPageActions.SetCity("and    ");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(26);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");


            SearchCustomerPageActions.SetCity("a W");
            SearchCustomerPageAssertions.AssertSearchResultsCount(7);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(7);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("3", "Antonio Moreno Taquer" + "\u00ED" + "a");

            SearchCustomerPageActions.SetCity("a   W");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }



        /// <summary>
        /// To test that no search results are retrieved when the search text in the city is deleted.
        /// </summary>
        [TestMethod]
        public void NoSearchResultsWhenSearchTextInCityIsDeletedTest()
        {
            SearchCustomerPageActions.SetCity("And");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(26);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");

            SearchCustomerPageActions.SetCity(String.Empty);
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }



        /// <summary>
        /// To test that the search functionality behaves properly when the search text in the city
        /// fields exceeds max limit.
        /// </summary>
        //[TestMethod]
        public void SearchTextInCityExceedsMaximumLimitTest()
        {
            string longString = null;
            foreach (string resourceName in this.GetType().Assembly.GetManifestResourceNames())
            {
                if (resourceName.Contains("LongString"))
                {
                    longString = new StreamReader(this.GetType().Assembly.GetManifestResourceStream(resourceName)).ReadLine();
                    break;
                }
            }

            SearchCustomerPageActions.SetCity(longString);
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }
    }
}
