//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RealTimeSearchQuickstart.FunctionalTests.Pages;
using RealTimeSearchQuickstart.FunctionalTests.Actions;
using RealTimeSearchQuickstart.FunctionalTests.Assertions;
using System.Threading;
using System.IO;

namespace RealTimeSearchQuickstart.FunctionalTests
{
    /// <summary>
    /// Tests for the search customer by company name functionality.
    /// </summary>
    [TestClass]
    public class SearchCustomerByCompanyNameTests
    {
        private SearchCustomerPage searchPage;


        [AssemblyInitialize]
        public static void AssemblyInit(TestContext context)
        {
            TestInfrastructure.StartWebServer();
        }

        /// <summary>
        /// Test Initialization:
        ///     Create a new browser
        ///     Browse to the search customer page
        ///     Validate whether the search customer page has been loaded
        ///     Set the target page for the search customer actions and assertions
        /// </summary>
        [TestInitialize]
        public void TestInit()
        {
            TestInfrastructure.CreateNewBrowserInstance();
            searchPage = new SearchCustomerPage();
            searchPage.Navigate();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation());
            SearchCustomerPageActions.TargetPage = searchPage;
            SearchCustomerPageAssertions.TargetPage = searchPage;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            TestInfrastructure.CloseBrowserInstance();
        }

        [AssemblyCleanup]
        public static void AssemblyCleanup()
        {
            TestInfrastructure.StopWebServer();
        }


        /// <summary>
        /// To test that the proper search results are fetched when a single character is entered in the 
        /// company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithASingleCharacterInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("O");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
        }



        /// <summary>
        /// To test that the proper search results are fetched when a single character is entered in the 
        /// company name field and the search button is clicked.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithCompanyNameByClickingSearchButtonTest()
        {
            SearchCustomerPageActions.SetCompanyName("O");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);

            SearchCustomerPageActions.ClickSearchButton();

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);
        }



        /// <summary>
        /// To test that the proper search results are fetched when two characters are entered in the 
        /// company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithTwoCharactersInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("O");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(64);

            SearchCustomerPageActions.SetCompanyName("Ol");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("10", "Bottom-Dollar Markets");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("91", "Wolski  Zajazd");
            SearchCustomerPageAssertions.AssertSearchResultsCount(8);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(8);
        }


        /// <summary>
        /// To test that the proper search results are fetched when multiple characters are entered in the 
        /// company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithMultipleCharactersInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("z");

            SearchCustomerPageAssertions.AssertSearchResultsCount(6);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(6);

            SearchCustomerPageActions.SetCompanyName("zi");

            SearchCustomerPageAssertions.AssertSearchResultsCount(3);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(3);

            SearchCustomerPageActions.SetCompanyName("zin");

            SearchCustomerPageAssertions.AssertSearchResultsCount(2);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(2);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("49", "Magazzini Alimentari Riuniti");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("62", "Queen Cozinha");

            SearchCustomerPageActions.SetCompanyName("zini");

            SearchCustomerPageAssertions.AssertSearchResultsCount(1);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(1);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("49", "Magazzini Alimentari Riuniti");
            SearchCustomerPageAssertions.AssertSearchResultNotPresentInGrid("62", "Queen Cozinha");
        }



        /// <summary>
        /// To test that the search functionality works as expected when special characters are used in 
        /// the company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithSpecialCharactersInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("zini");
          
            SearchCustomerPageAssertions.AssertSearchResultsCount(1);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(1);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("49", "Magazzini Alimentari Riuniti");
            SearchCustomerPageAssertions.AssertSearchResultNotPresentInGrid("62", "Queen Cozinha");

            SearchCustomerPageActions.SetCompanyName(String.Empty);
            Thread.Sleep(TestInfrastructure.DefaultTimeout);

            foreach (string inputWithSpecialCharacters in TestInfrastructure.EmbedSpecialCharactersInString("zini"))
            {
                SearchCustomerPageActions.SetCompanyName(inputWithSpecialCharacters);
                SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
                SearchCustomerPageAssertions.AssertErrorMessageIsDisplayed("Invalid search query string");
            }

            SearchCustomerPageActions.SetCompanyName(String.Empty);
            SearchCustomerPageActions.SetCompanyName(@"'a");
            SearchCustomerPageAssertions.AssertSearchResultsCount(2);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(2);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("40", "La corne d'abondance");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("41", "La maison d'Asie"); 
        }


        /// <summary>
        /// To test that the search functionality works as expected when unicode characters are used in 
        /// the company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithUnicodeCharactersInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("\u00F6");

            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
            SearchCustomerPageAssertions.AssertErrorMessageIsDisplayed("Invalid search query string");         
        }


        /// <summary>
        /// To test that no search results are returned when only blankspaces are entered in the company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithOnlyBlankSpacesInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("    ");
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }


        /// <summary>
        /// To test that search functionality works as expected when blankspaces are present in the search text
        /// in the company name field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithBlankSpacesInSearchTextInCompanyNameTest()
        {
            SearchCustomerPageActions.SetCompanyName("       Zin");
            SearchCustomerPageAssertions.AssertSearchResultsCount(2);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(2);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("49", "Magazzini Alimentari Riuniti");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("62", "Queen Cozinha");

            SearchCustomerPageActions.SetCompanyName("Zini    ");
            SearchCustomerPageAssertions.AssertSearchResultsCount(1);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(1);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("49", "Magazzini Alimentari Riuniti");

            SearchCustomerPageActions.SetCompanyName("D H");
            SearchCustomerPageAssertions.AssertSearchResultsCount(1);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(1);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("16", "Consolidated Holdings");
            
            SearchCustomerPageActions.SetCompanyName("D   H");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
         }



        /// <summary>
        /// To test that no search results are retrieved when the search text in the company field is deleted.
        /// </summary>
        [TestMethod]
        public void NoSearchResultsWhenSearchTextInCompanyNameIsDeletedTest()
        {
            SearchCustomerPageActions.SetCompanyName("Zin");
            SearchCustomerPageAssertions.AssertSearchResultsCount(2);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(2);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("49", "Magazzini Alimentari Riuniti");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("62", "Queen Cozinha");

            SearchCustomerPageActions.SetCompanyName(String.Empty);
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();            
        }



        /// <summary>
        /// To test that the search functionality behaves properly when the search text in the company name
        /// fields exceeds max limit.
        /// </summary>
        //[TestMethod]
        public void SearchTextInCompanyNameExceedsMaximumLimitTest()
        {
            string longString = null;
            foreach (string resourceName in this.GetType().Assembly.GetManifestResourceNames())
            {
                if (resourceName.Contains("LongString"))
                {
                    longString = new StreamReader(this.GetType().Assembly.GetManifestResourceStream(resourceName)).ReadLine();
                    break;
                }
            }

            SearchCustomerPageActions.SetCompanyName(longString);           
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }


    }
}
