//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RealTimeSearchQuickstart.FunctionalTests.Pages;
using RealTimeSearchQuickstart.FunctionalTests.Actions;
using RealTimeSearchQuickstart.FunctionalTests.Assertions;
using System.Threading;
using System.IO;

namespace RealTimeSearchQuickstart.FunctionalTests
{
    /// <summary>
    /// Tests for the search customer by postal code functionality.
    /// </summary>
    [TestClass]
    public class SearchCustomerByPostalCodeTests
    {

        private SearchCustomerPage searchPage;


        /// <summary>
        /// Test Initialization:
        ///     Create a new browser
        ///     Browse to the search customer page
        ///     Validate whether the search customer page has been loaded
        ///     Set the target page for the search customer actions and assertions
        /// </summary>
        [TestInitialize]
        public void TestInit()
        {
            TestInfrastructure.CreateNewBrowserInstance();
            searchPage = new SearchCustomerPage();
            searchPage.Navigate();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation());
            SearchCustomerPageActions.TargetPage = searchPage;
            SearchCustomerPageAssertions.TargetPage = searchPage;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            TestInfrastructure.CloseBrowserInstance();
        }



        /// <summary>
        /// To test that the proper search results are fetched when a single character is entered in the 
        /// postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithASingleCharacterInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("0");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("1", "Alfreds Futterkiste");

        }

        /// <summary>
        /// To test that the proper search results are fetched when a search text is entered in the 
        /// postal code field and the search button is clicked.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithPostalCodeByClickingSearchButtonTest()
        {
            SearchCustomerPageActions.SetPostalCode("0");
            SearchCustomerPageActions.ClickSearchButton();

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("1", "Alfreds Futterkiste");
        }


        /// <summary>
        /// To test that the proper search results are fetched when two characters are entered in the 
        /// postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithTwoCharactersInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("0");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);

            SearchCustomerPageActions.SetPostalCode("03");

            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(13);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("14", "Chop-suey Chinese");
        }



        /// <summary>
        /// To test that the proper search results are fetched when two characters are entered in the 
        /// postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithMultipleCharactersInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("0");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);

            SearchCustomerPageActions.SetPostalCode("03");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(13);

            SearchCustomerPageActions.SetPostalCode("034");
            SearchCustomerPageAssertions.AssertSearchResultsCount(6);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(6);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("25", "Frankenversand");
        }



        /// <summary>
        /// To test that a proper message is displayed and the search functionality is not allowed
        /// when special characters are used in the postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithSpecialCharactersInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("0");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("14", "Chop-suey Chinese");

            SearchCustomerPageActions.SetPostalCode(String.Empty);
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            
            foreach (string inputWithSpecialCharacters in TestInfrastructure.EmbedSpecialCharactersInString("0"))
            {
                SearchCustomerPageActions.SetPostalCode(inputWithSpecialCharacters);
                SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
                SearchCustomerPageAssertions.AssertErrorMessageIsDisplayed("Invalid search query string");
            }

            SearchCustomerPageActions.SetPostalCode(String.Empty);
            SearchCustomerPageActions.SetPostalCode(@"'0");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }


        /// <summary>
        /// To test that a proper message is displayed and the search functionality is not allowed
        /// when unicode characters are used in the postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithUnicodeCharactersInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("\u00F6");
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
            SearchCustomerPageAssertions.AssertErrorMessageIsDisplayed("Invalid search query string");
        }


        /// <summary>
        /// To test that no search results are returned when only blankspaces are entered in the postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithOnlyBlankSpacesInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("    ");
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }


        /// <summary>
        /// To test that search functionality works as expected when blankspaces are present in the search text
        /// in the postal code field.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithBlankSpacesInSearchTextInPostalCodeTest()
        {
            SearchCustomerPageActions.SetPostalCode("       0");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("14", "Chop-suey Chinese");

            SearchCustomerPageActions.SetPostalCode("0    ");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("14", "Chop-suey Chinese");


            SearchCustomerPageActions.SetPostalCode("9 8");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }



        /// <summary>
        /// To test that no search results are retrieved when the search text in the postal code is deleted.
        /// </summary>
        [TestMethod]
        public void NoSearchResultsWhenSearchTextInPostalCodeIsDeletedTest()
        {
            SearchCustomerPageActions.SetPostalCode("0");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(53);
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("12", "Cactus Comidas para llevar");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("14", "Chop-suey Chinese");

            SearchCustomerPageActions.SetPostalCode(String.Empty);
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }



        /// <summary>
        /// To test that the search functionality behaves properly when the search text in the postal code
        /// field exceeds max limit.
        /// </summary>
        //[TestMethod]
        public void SearchTextInPostalCodeExceedsMaximumLimitTest()
        {
            string longString = null;
            foreach (string resourceName in this.GetType().Assembly.GetManifestResourceNames())
            {
                if (resourceName.Contains("LongString"))
                {
                    longString = new StreamReader(this.GetType().Assembly.GetManifestResourceStream(resourceName)).ReadLine();
                    break;
                }
            }

            SearchCustomerPageActions.SetPostalCode(longString);
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }
    }
}
