//===============================================================================
// Microsoft patterns & practices
// Web Client Software Factory
//-------------------------------------------------------------------------------
// Copyright (C) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//-------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using RealTimeSearchQuickstart.FunctionalTests.Pages;
using RealTimeSearchQuickstart.FunctionalTests.Actions;
using RealTimeSearchQuickstart.FunctionalTests.Assertions;
using System.Threading;
using System.IO;


namespace RealTimeSearchQuickstart.FunctionalTests
{
    /// <summary>
    /// Tests for the search customer by state functionality.
    /// </summary>
    [TestClass]
    public class SearchCustomerByStateTests
    {

        private SearchCustomerPage searchPage;

                

        /// <summary>
        /// Test Initialization:
        ///     Create a new browser
        ///     Browse to the search customer page
        ///     Validate whether the search customer page has been loaded
        ///     Set the target page for the search customer actions and assertions
        /// </summary>
        [TestInitialize]
        public void TestInit()
        {
            TestInfrastructure.CreateNewBrowserInstance();
            searchPage = new SearchCustomerPage();
            searchPage.Navigate();
            Thread.Sleep(TestInfrastructure.DefaultTimeout);
            Assert.IsTrue(searchPage.ValidateLocation());
            SearchCustomerPageActions.TargetPage = searchPage;
            SearchCustomerPageAssertions.TargetPage = searchPage;
        }

        [TestCleanup]
        public void TestCleanup()
        {
            TestInfrastructure.CloseBrowserInstance();
        }


        /// <summary>
        /// To test that the proper search results are fetched when a state is selected.
        /// </summary>
        [TestMethod]
        public void SearchCustomerWithStateSelectedTest()
        {
            SearchCustomerPageActions.SelectState("Washington");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("27", "Franchi S.p.A.");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(23);
        }


        /// <summary>
        /// To test that the search results are refreshed immediately when the selected state is changed.
        /// </summary>
        [TestMethod]
        public void SearchCustomerResultsRefreshedWhenSelectedStateIsChangedTest()
        {
            SearchCustomerPageActions.SelectState("Washington");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("27", "Franchi S.p.A.");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(23);

            SearchCustomerPageActions.SelectState("Oregon");
            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("5", "Berglunds snabbk" + "\u00F6" + "p");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(27);
        }


        /// <summary>
        /// To test that no search results are returned when no state is selected.
        /// </summary>
        [TestMethod]
        public void SearchCustomerResultsAreNotRetrievedWhenNoStateIsSelectedTest()
        {
            SearchCustomerPageActions.SelectState("Washington");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("27", "Franchi S.p.A.");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(23);

            SearchCustomerPageActions.SelectState(String.Empty);
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();

            SearchCustomerPageActions.ClickSearchButton();
            SearchCustomerPageAssertions.AssertSearchResultGridDoesNotExist();
        }


        /// <summary>
        /// To test that no search results are returned when the chosen state does not match any of 
        /// the search results.
        /// </summary>
        [TestMethod]
        public void SearchCustomerResultsAreNotRetrievedWhenSelectedStateIsNotPresentTest()
        {
            SearchCustomerPageActions.SelectState("North Dakota");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();

            SearchCustomerPageActions.SelectState("Washington");

            SearchCustomerPageAssertions.AssertSearchResultPresentInGrid("27", "Franchi S.p.A.");
            SearchCustomerPageAssertions.AssertSearchResultsCount(10);
            SearchCustomerPageAssertions.AssertSearchResultsTotalCount(23);

            SearchCustomerPageActions.SelectState("New Hampshire");
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();

            SearchCustomerPageActions.ClickSearchButton();
            SearchCustomerPageAssertions.AssertNoSearchResultHasBeenRetrieved();
        }


    }
}
